/*
 * Created on Oct 14, 2003
 */
package jrequest.base;

/**
 * StateAspect
 * 
 * @author oliver
 * @since  Oct 14, 2003
 */
public aspect StateAspect {
    
    // ---  P O I N T C U T S  ------------------------------------------------

    pointcut executeSetAccepted():
        execution(public void jrequest.base.State.setAccepted());
    
    pointcut executeSetAssigned():
        execution(public void jrequest.base.State.setAssigned());
    
    pointcut executeSetUnderWork():
        executeSetAccepted() || executeSetAssigned();

    pointcut executeAnyConstructor():
        execution(public State.new(..));

    pointcut callAnyConstructor():
        call(public State.new(..));

    pointcut executeConstructor(State s):
        execution(public State.new(..)) && this(s);

    pointcut callSetAccepted():
        call(public void jrequest.base.State.setAccepted());
    
    pointcut initState():
        initialization(State.new(..));

    pointcut preinitState():
        preinitialization(State.new(..));
        
    pointcut staticinitState():
        staticinitialization(State);
    
    pointcut handleRuntimeException():
        handler(RuntimeException);



    // ---  A D V I C E S  ----------------------------------------------------

    before(): executeSetAccepted() {
        System.out.println("executing " + thisJoinPoint.getThis());
    }

    before(State s): executeConstructor(s) {
        System.out.println("history = " + s.history);
    }
    
    before() : callSetAccepted() {
        System.out.println("called from " + thisJoinPoint.getThis());
    }
    
    before(): executeSetAccepted() {
        System.out.println("executing in state " + thisJoinPoint.getThis());
    }
    
    before() : initState() {
        System.out.println("initState: " + thisJoinPoint.getThis());
    }
    
    after() : preinitState() {
        System.out.println("preinitState: " + thisJoinPoint.getThis());
    }
    
    after() : staticinitState() {
        System.out.println("staticinitState: " + thisJoinPointStaticPart);
    }
    
    before() : handleRuntimeException() {
        System.err.println("exception in " + thisJoinPoint.getThis());
    }

    
    
    /////   Im Banne des Joinpoints   /////
    
    before(): callSetAccepted() {
        System.out.println("1 thisJoinPoint = " + thisJoinPoint);
        System.out.println("1 called from " + thisJoinPoint.getThis());
    }

    /////   this   /////
    
    pointcut callSetAccepted2(Request request):
        call(public void State.setAccepted()) && this(request);

    before(Request request): callSetAccepted2(request) {
        System.out.println("2 thisJoinPoint = " + thisJoinPoint);
        System.out.println("2 called from " + request);
    }

    /////   Kontrollfluss-basierte Bezeichner / cflow   /////
    
    pointcut callSetAssignedInRequest1():
        call(public void State.setAssigned())
            && cflow(call(public void Request.*(..)));

    before() : callSetAssignedInRequest1() {
        System.out.println(thisJoinPoint
            + " from " + thisJoinPoint.getThis());
    }
    
    before() : (call(public void State.setAssigned())
            || call(public void Request.*assign*(..)))
        && cflowbelow(call(public void Request.reassignTo(..))) {
        System.err.println("cflowbelow: " + thisJoinPoint);
    }
    before() : (call(public void State.setAssigned())
            || call(public void Request.*assign*(..)))
        && cflow(call(public void Request.reassignTo(..))) {
        System.err.println("cflow: " + thisJoinPoint);
    }
    
    pointcut callRequestMethod2():
        call(public void Request.*(..));

    pointcut callSetAssignedInRequest2():
        call(public void State.setAssigned())
            && cflow(callRequestMethod2());

    pointcut callRequestMethod3():
        call(public void Request.assignTo(User))
            || call(public void Request.reassignTo(User));

    pointcut callSetAssignedInRequest3():
        call(public void State.setAssigned())
            && cflow(callRequestMethod3());
    
    /////   Programmtext-basierte Bezeichner / withincode   /////
    
    pointcut callSetAssignedInRequest4():
        call(public void State.setAssigned())
            && cflow(call(public void Request.assignTo(..)));
    
    before() : callSetAssignedInRequest4() {
        System.out.println("<callSetAssignedInRequest4>");
    }

    pointcut callSetAssignedInRequest5():
        call(public void State.setAssigned())
        && withincode(public void Request.assignTo(..));
    
    @org.aspectj.lang.annotation.SuppressAjWarnings
    before() : callSetAssignedInRequest5() {
        System.out.println("<callSetAssignedInRequest5>");
    }

}
