/*
 * Created on 13.02.2005
 *
 * $Id: FareCalculator.java,v 1.1 2005/06/24 13:56:05 boehm Exp $
 */
package space;

import java.util.Date;

import space.annotation.*;

/**
 * Gebuehren-Rechner, als Singleton realisiert.<br>
 * (Anm.: es geht auch ohne Singleton, als reine statische Klasse.
 * Aber dann haette ich kein Beispiel fuer "@Singleton" ;-)
 * 
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 13.02.2005
 * @version $revision$
 */
@Critical
@Secret
public class FareCalculator {
    
    static private FareCalculator instance = null;
    
    /** Preis pro Meile */
    @Constant
    @Tariff
    private double mileageFare = 0.2;
    
    /** Preis pro Minute */
    @Constant
    private double minuteFare = 0.5;
    
    /** Grundgebuehr */
    @Constant
    private Integer basicFee = 2;
    
    static public FareCalculator getInstance() {
        if (instance == null) {
            instance = new FareCalculator();
        }
        return instance;
    }
    
    protected FareCalculator() {}
    
    public double getFare(Date startDate, Date endDate, long mileage) {
        long minutes = (endDate.getTime() - startDate.getTime() + 59999) / 60000;
        return getFare(minutes, mileage);
    }
    
    public double getFare(long minutes, long mileage) {    
        return basicFee + minutes * minuteFare + mileage * mileageFare;
    }

}
