/*
 * Created on 09.11.2004
 *
 * $Id: AspectHelper.java,v 1.1 2005/06/24 13:56:06 boehm Exp $
 */
package hello;

import java.lang.reflect.Modifier;

import org.aspectj.lang.*;
import org.aspectj.lang.reflect.SourceLocation;

/**
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 09.11.2004
 * @version $revision$
 */
public class AspectHelper {
    
    public static void printJoinPoint(JoinPoint point) {
        System.out.println("some internals of -> " + point);
        System.out.println("\targs:       " + point.getArgs());
        System.out.println("\tkind:       " + point.getKind());
        System.out.println("\tsignature:  " + point.getSignature());
        System.out.println("\tlocation:   " + point.getSourceLocation());
        System.out.println("\tstaticPart: " + point.getStaticPart());
        System.out.println("\ttarget:     " + point.getTarget());
        System.out.println("\tthis:       " + point.getThis());
        System.out.println("\tstring:     " + point.toString());
        System.out.println("\t    long:   " + point.toLongString());
        System.out.println("\t    short:  " + point.toShortString());
    }
    
    public static void printStaticJoinPoint(JoinPoint.StaticPart point) {
        System.out.println("some static internals of -> " + point);
        System.out.println("\tkind:       " + point.getKind());
        System.out.println("\tsignature:  " + point.getSignature());
        System.out.println("\tlocation:   " + point.getSourceLocation());
        System.out.println("\tstring:     " + point.toString());
        System.out.println("\t    long:   " + point.toLongString());
        System.out.println("\t    short:  " + point.toShortString());
    }
    
    /**
     * Print the signature of a JoinPoint.
     * @param point
     * @author <a href="mailto:boehm@javatux.de">oliver</a>
     * @since  09.11.2004
     */
    public static void printSignature(JoinPoint point) {
        printSignature(point.getSignature());
    }
    
    public static void printSignature(Signature sig) {
        System.out.println("some internals of -> " +  sig);
        System.out.println("decl. type: " + sig.getDeclaringType());
        printModifiers(sig.getModifiers());
        System.out.println("name:       " + sig.getName());
        System.out.println("string:     " + sig.toString());
        System.out.println("    long:   " + sig.toLongString());
        System.out.println("    short:  " + sig.toShortString());
    }
    
    public static void printModifiers(int m) {
        System.out.println("modifiers:  " + Modifier.toString(m));
    }

    /**
     * Print the source loacation of a JoinPoint.
     * @param point
     * @author <a href="mailto:boehm@javatux.de">oliver</a>
     * @since  09.11.2004
     */
    public static void printSourceLocation(JoinPoint point) {
        printSourceLocation(point.getSourceLocation());
    }
    
    public static void printSourceLocation(SourceLocation loc) {
        System.out.println("some internals of -> " + loc);
        System.out.println("file: " + loc.getFileName());
        System.out.println("line: " + loc.getLine());
        System.out.println("type: " + loc.getWithinType());
    }

}
