/*
 * Created on 07.04.2005
 *
 * $Id: SubjectAspect.aj,v 1.1 2005/06/24 13:56:07 boehm Exp $
 */
package pattern.observer;

public aspect SubjectAspect {
    
    private SubjectObserverSet Subject.observers = new SubjectObserverSet();

    public void Subject.addObserver(SubjectObserver observer) {
        observers.add(observer);
    }

    public void Subject.notifyObservers() {
        observers.notifyObservers(this);
    }

    
    
    /**
     * call notifyObservers() automatically each an attribute marked as @Observe
     * has changed (but only inside a @Observe method)
     */
//    after(Subject subject) : set(@Observe Object+ Subject+.*)
//            && withincode(@Observe * Subject+.*(..))
//            && this(subject) {
//        subject.notifyObservers();
//    }

    /**
     * call notifyObservers() automatically each time method marked as @Observe
     * is executed.
     */
//    after(Subject subject) : execution(@Observe * Subject+.*(..))
//            && this(subject) {
//        subject.notifyObservers();
//    }
    
    /**
     * call notifyObservers() automatically if an attribut has changed
     * except for those which should be ignored.
     */
    after(Subject subject) : set(!@Ignore Object+ Subject+.*)
            && !withincode(@Ignore * Subject+.*(..))
            && this(subject) {
        subject.notifyObservers();
    }
    
}
