/*
 * Created on 23.05.2005
 *
 * $Id: Robot3.java,v 1.1 2005/06/24 13:56:06 boehm Exp $
 */
package web;

import java.io.*;
import java.io.IOException;
import java.net.*;
import java.net.Socket;


/**
 * Ein einfacher Robot, der ausgeschickt wird, um das Web zu erforschen.
 * Die Implementierung ist sehr einfach gehalten und verwendet keine Techniken
 * wie Pooling oder Caching. Diese werden ueber SocketPoolingAspect
 * hinzugefuegt.
 * 
 * @see SocketPoolingAspect
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 26.05.2005
 * @version $revision$
 */
public class Robot3 {
    
    public String hostname = null;
    
    public Robot3() {
        this("localhost");
    }
    
    public Robot3(String hostname) {
        this.hostname = hostname;
    }
    
    /**
     * Ein einfacher Verbindungsaufbau, um festzustellen, ob der
     * gewuenschte Webserver erreichbar ist.
     * 
     * @return Antwort des Webservers bzw. Fehlermeldung
     */
    public String ping() {
        try {
            long t0 = System.currentTimeMillis();
            Socket socket = new Socket(hostname, 80);
            if (socket != null) {
                // hier jetzt auf den Socket zugreifen
                send("\n", socket.getOutputStream());
                // und die Antwort des Servers interpretieren
                // ...
                socket.close();
                long t1 = System.currentTimeMillis();
                return this.hostname + ":80 is alive time=" + (t1-t0) + " ms";
            }
        } catch (UnknownHostException e) {
            return "unknown host " + hostname;
        } catch (IOException e) {
            return this.hostname + ":80: " + e;
        }
        return "no answer from " + this.hostname;
    }
    
    public String ping(int n) {
        String answer = null;
        long t0 = System.currentTimeMillis();
        for (int i = 0; i < n; i++) {
            answer = ping();
            System.out.println(answer);
        }
        long t1 = System.currentTimeMillis();
        return n + " pings in " + (t1 -t0) + " ms - " + answer;
    }

    private void send(String s, OutputStream out) throws IOException {
        out.write(s.getBytes());
    }

    public String toString() {
        return this.getClass().getName() + " for " + this.hostname;
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        Robot3 robot = new Robot3();
        if (args.length > 0) {
            robot.hostname = args[0];
        }
        System.out.println(robot + " started...");
        System.out.println(robot.ping());
        System.out.println(robot.ping(3));
    }

}
