/*
 * Created on 26.05.2005
 *
 * $Id: SocketPool.java,v 1.1 2005/06/24 13:56:06 boehm Exp $
 */
package web;

import java.io.IOException;
import java.net.*;
import java.util.*;

public class SocketPool {
    
    /** der eigentliche Pool mit unbegrenzter Groesse */
    private Map pool = null;
    
    /** Default-Port fuer Web-Server */
    private int port = 80;
    
    public SocketPool() {
        this(80);
    }
    
    public SocketPool(int port) {
        this.port = port;
        this.pool = new HashMap();
    }
    
    /**
     * Falls nur eine begrenzte Anzahl von Sockets geoeffnet werden koennen,
     * sollten wir hier vielleicht bei einer Socket-Exception alle offenen
     * Sockets schliessen und es nochmals versuchen.
     * 
     * @param host
     * @return
     * @throws UnknownHostException
     * @throws IOException
     */
    public Socket getSocket(InetAddress address) throws IOException {
        Socket socket = (Socket) pool.get(address);
        if (socket == null) {
            socket = new Socket(address, this.port);
        }
        return socket;
    }
    
    public Socket getSocket(String host) throws UnknownHostException,
            IOException {
        return getSocket(InetAddress.getByName(host));
    }
    
    /**
     * Andere Ports ausser dem voreingestellten Default-Port werden vorerst
     * noch nicht gepoolt.
     * 
     * @param address
     * @param port
     * @return
     * @throws IOException
     */
    public Socket getSocket(InetAddress address, int port) throws IOException {
        if (port == this.port) {
            return this.getSocket(address);
        } else {
            return new Socket(address, this.port);
        }
    }
    
    public Socket getSocket(String host, int port) throws UnknownHostException,
            IOException {
        return getSocket(InetAddress.getByName(host), port);
    }
    
    /**
     * Nur die Sockets mit dem voreingestellten Default-Port werden wieder
     * in den Pool zurueckgestellt.
     * @param socket
     */
    public void putSocket(Socket socket) throws IOException {
        if (socket.getPort() == this.port) {
            pool.put(socket.getInetAddress(), socket);
        } else {
            socket.close();
        }
    }
    
    

}
