/*
 * Created on 27.05.2005
 *
 * $Id: SocketPoolingAspect.aj,v 1.1 2005/06/24 13:56:06 boehm Exp $
 */
package web;

import java.io.IOException;
import java.net.*;

public aspect SocketPoolingAspect {
    
    /** Anlegen des SocketPools */
    private SocketPool socketPool = new SocketPool(80);
    
    /**
     * Robot1 and Robot2 werden von der Instrumentierung ausgeschlossen.
     * Sie dienen nur zu Demo- und Vergleichszwecke fuer die konventionelle
     * Java-Programmierung.
     * Und natuerlich muss auch SocketPool ausgenommen werden -- sonst droht
     * eine StackOverflowException!
     */
    pointcut excluded() :
        !within(SocketPool) && !within(Robot1) && !within(Robot2);
    
    /**
     * Pointcut fuer das Oeffnen bzw. Anlegen eines Sockets
     */
    pointcut socketCreation(String host, int port) :
        call(public Socket.new(*, int))
        && args(host, port)
        && excluded();
    
    /**
     * Pointcut fuer das Schliessen eines Sockets
     */
    pointcut socketClose(Socket socket) :
        call(public void Socket.close())
        && target(socket)
        && excluded();
    
    /**
     * Das Oeffnen eines Sockets erfolgt nun ueber den socketPool.
     * Falls dies fehlschlaegt, wird der Original-Pointcut aufgerufen.
     */
    Socket around(String host, int port) 
            throws UnknownHostException, IOException 
            : socketCreation(host, port) {
        Socket socket = socketPool.getSocket(host, port);
        if (socket == null) {
            return proceed(host, port);
        } else {
            return socket;
        }
    }
    
    /**
     * Beim Schliessen des Sockets wandert er wieder in den Pool
     */
    void around(Socket socket) throws IOException : socketClose(socket) {
        socketPool.putSocket(socket);
    }

}
