package verwaltung;

import java.util.*;

/*
 * Created on Jul 5, 2004
 */

/**
 * @author oliver
 * @since Jul 5, 2004
 */
public class Person {
    
    public String name;
    public String email;
    public Date geburtstag;
    
    public Person(String name) {
        this.name = name;
    }
    
    public Person(String name, Date geburtstag) {
        this.name = name;
        this.geburtstag = geburtstag;
    }
    
    public String getName() {
        return this.name;
    }
    
    /**
     * Diese Methode überprüft, ob der übergebene Parameter s
     * aus gültigen Zeichen besteht und das '@'-Zeichen enthält.
     * <br>
     * Unter JDK 1.3 kann statt s.matches("muster") auch einfach
     * die Abrage (s.indexOf('@') > 0) verwendet werden, um die
     * Existenz des '@'-Zeichens zu testen.
     * 
     * @param s
     * @author <a href="mailto:boehm@javatux.de">oliver</a>
     * @since  16.11.2004
     */
    public void setEmail(String s) {
        if (s.matches("[a-zA-Z_0-9.\\-]+@[a-zA-Z_0-9.\\-]+")) {
            this.email = s;
        } else {
            throw new IllegalArgumentException("invalid email address " + s);
        }
    }
    
    public String getEmail() {
        return this.email;
    }

//    public boolean isVolljaehrig() {
//        // Volljaehrigkeitszeitpunkt ermitteln
//        Calendar volljaehrigkeit = new GregorianCalendar();
//        volljaehrigkeit.setTime(this.geburtstag);
//        volljaehrigkeit.add(Calendar.YEAR, 18);
//        
//        // Volljaehrigkeit beginnt um 0:00
//        volljaehrigkeit.set(Calendar.HOUR_OF_DAY, 0);
//        volljaehrigkeit.set(Calendar.MINUTE, 0);
//        volljaehrigkeit.set(Calendar.SECOND, 0);
//        volljaehrigkeit.set(Calendar.MILLISECOND, 0);
//        
//        // mit aktuellem Datum vergleichen
//        Calendar now = new GregorianCalendar();
//        return now.after(volljaehrigkeit);
//    }

    public boolean isVolljaehrig() {
        Calendar now = new GregorianCalendar();
        return now.after(this.getVolljaehrigkeitsZeitpunkt());
    }

    /**
     * @return
     */
    private Calendar getVolljaehrigkeitsZeitpunkt() {
        Calendar volljaehrigkeit = getGeburtstagAsCalendar();
        volljaehrigkeit.add(Calendar.YEAR, 18);        
        setUhrzeitAufNull(volljaehrigkeit);
        return volljaehrigkeit;
    }

    /**
     * @return
     */
    private Calendar getGeburtstagAsCalendar() {
        Calendar geburt = new GregorianCalendar();
        geburt.setTime(this.geburtstag);
        return geburt;
    }

    /**
     * Setzt die Uhrzeit auf 0:00
     * @param calendar
     */
    private void setUhrzeitAufNull(Calendar calendar) {
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
    }

    public String toString() {
        return "mein Name ist " + this.name + ", mailto:" + this.email;
    }
    
    public static void main(String[] args) {
        Person hase = new Person("Hase");
        System.out.println(hase.toString());
    }

}
