/*
 * Created on 30.03.2005
 *
 * $Id$
 */
package web;

import java.io.*;
import javax.servlet.http.*;

import org.aspectj.lang.*;


public aspect LoginServletTestAspect {

    pointcut belowTest() : 
        cflowbelow(execution(* LoginServletTest.*(..)));
    
    /**
     * Der ContentType interessiert hier nicht fuer den Test.
     * Wir ignorieren ihn einfach.
     */
    void around() : call(void HttpServletResponse.setContentType(String))
                    && belowTest() {
        ignore(thisJoinPoint);
    }
    
    /**
     * Wir haben unseren eigenen PrintWriter, der von LoginServletTest
     * gestellt wird.
     */
    PrintWriter around() : call(PrintWriter HttpServletResponse.getWriter())
                           && belowTest() {
        ignore(thisJoinPoint);
        return LoginServletTest.out;
    }
    
    /**
     * Fuer die Parameter-Abfrage geben wir die Werte zurueck, die
     * wir vom LoginServletTest bekommen haben.
     */
    String around(String name) : 
            call(* HttpServletRequest.getParameter(String))
            && args(name)
            && belowTest() {
        if (name.equals("loginname")) {
            return(LoginServletTest.username);
        } else if (name.equals("password")) {
            return(LoginServletTest.password);
        } else {
            return null;
        }
    }
    
    private void ignore(JoinPoint joinpoint) {
        System.out.println("ignoring " + joinpoint.toShortString()
                + " for testing");
    }

}
