/*
 * Created on 15.03.2005
 *
 * $Id$
 */
package spiel;



import java.lang.reflect.*;

import org.aspectj.lang.*;

import junit.framework.TestCase;

import spiel.annotation.*;



public aspect TestAspekt {
    
    private static int testLevel;
    
    static {
        try {
            testLevel = Integer.parseInt(System.getProperty("test.level"));
        } catch (NumberFormatException e) {
            testLevel = 3;  // all tests
        }
    }
    
    pointcut testMethods() : execution(public void TestCase+.test*());
    
    void around() : testMethods() {
        Signature sig = thisJoinPoint.getSignature();
        if (getTestLevel(sig) <= testLevel) {
            proceed();
            //System.out.println(sig.toShortString() + " executed!");
        } else {
            System.out.println(sig.toShortString() + " NOT executed!");
        }
    }
    
    private int getTestLevel(Signature sig) {
        Test test = getTestAnnotation(sig);
        if (test != null) {
            //return test.level();
            return test.value().ordinal();
        } else {
            return 0;
        }
    }
    
    /**
     * Ermittelt ueber die Signatur die Test-Annotation der
     * Test-Methode.
     */
    private Test getTestAnnotation(Signature sig) {
        try {
            Method method = getMethod(sig);
            return method.getAnnotation(Test.class);
        } catch (NoSuchMethodException shouldNotHappen) {
            return null;
        }
    }
    
    /**
     * Ermittelt aus der Signatur die entsprechende Methode.
     * Da JUnit-Testmethoden keinen Parameter haben, werden diese
     * auch nicht aus der Signature heraus ermittelt, sondern
     * stattdessen ein leeres Class-Array uebergeben.
     */
    private Method getMethod(Signature sig) throws NoSuchMethodException {
        Class cl = sig.getDeclaringType();
        String name = sig.getName();
        return cl.getDeclaredMethod(name, new Class[0]);
    }
    
}
