/*
 * Created on 15.03.2005
 *
 * $Id$
 */
package spiel;

import junit.framework.TestCase;
import spiel.annotation.*;

public class WuerfelTest extends TestCase {
    
    private Wuerfel wuerfel = null;

    public static void main(String[] args) {
        junit.textui.TestRunner.run(WuerfelTest.class);
    }

    protected void setUp() throws Exception {
        wuerfel = new Wuerfel();
    }

    /**
     * Die Augenzahl nach 10 Wuerfen sollte irgendwo so zwischen 20 und 50
     * liegen (idealerweise bei 35). Liegt es ausserhalb dieses Bereichs ist
     * irgendetwas mit dem Zufallszahlengenerator faul.
     */
    //@Test(level=1)
    @Test(Level.QUICK)
    public void testRolleWenig() {
        int sum = rolleWuerfel(10);
        assertTrue(sum > 20);
        assertTrue(sum < 50);
    }
    
    /**
     * Nach 1000 Wuerfen sollte die geworfenen Augenzahl schon irgendwo
     * zwischen 3000 und 4000 liegen, sonst ist der Wuerfel gezinkt.
     */
    //@Test(level=2)
    @Test(Level.NORMAL)
    public void testRolleOft() {
        int sum = rolleWuerfel(1000);
        assertTrue(sum > 3000);
        assertTrue(sum < 4000);
    }
    
    /**
     * Jetzt sind wir etwas pingeliger: Nach 1 Mio Wuerfen wird eine Augenzahl
     * zwischen 3,4 und 3,6 Millionen erwartet.
     */
    //@Test(level=3)
    @Test(Level.PEDANTIC)
    public void testRolleVerdammtOft() {
        int sum = rolleWuerfel(1000000);
        assertTrue(sum > 3400000);
        assertTrue(sum < 3600000);
    }

    public void testGetDurchschnitt() {
        double d = rolleWuerfel(50) / 50.0;
        assertEquals(d, wuerfel.getDurchschnitt(), 0.01);
    }

    private int rolleWuerfel(int n) {
        int sum = 0;
        for (int i = 0; i < n; i++) {
            sum += wuerfel.rolle();
        }
        System.out.println("Summe nach " + n + " Wuerfen: " + sum);
        return sum;
    }
    
    //@Test(level=3)
    @Test(Level.PEDANTIC)
    public void testAspektPerformance() {
        int n = 1000000;
        long t0 = System.currentTimeMillis();
        for (int i = 0; i < n; i++) {
            testDummy();
        }
        long t1 = System.currentTimeMillis();
        System.out.println("1 x testDummy(): " + (t1-t0)*1000.0/n + " ns");
        for (int i = 0; i < n; i++) {
            dummy();
        }
        long t2 = System.currentTimeMillis();
        System.out.println("1 x dummy():     " + (t2-t1)*1000.0/n + " ns");
    }
    
    public void testDummy() {}
    
    public void dummy() {
//        assertEquals(1, 1);
//        assertEquals(2, 2);
//        assertEquals(3, 3);
//        assertEquals(4, 4);
//        assertEquals(5, 5);
//        assertEquals(6, 6);
    }
    
    public void testLevel() {
        Level level = Level.QUICK;
        assertEquals(1, level.ordinal());
    }

}
