/*
 * $Id: ComputerPlayer.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 *
 * Copyright (c) 2001 Oliver Boehm (mailto:boehm@2xp.de)
 * http://www.2xp.de
 * All rights reserved.
 *
 * -----------------------------------------
 *
 * $Author: boehm $
 * $Date: 2005/06/24 13:56:04 $
 * $Revision: 1.1 $
 *
 * $Log: ComputerPlayer.java,v $
 * Revision 1.1  2005/06/24 13:56:04  boehm
 * Einspielen der Beispiele und Uebungen inkl. HTML-Seiten
 *
 * Revision 1.3  2002/03/27 09:22:56  oliver
 * minor improvements
 *
 * Revision 1.2  2001/12/27 19:12:50  oliver
 * ComputerPlayer.main() extended
 *
 * Revision 1.1  2001/12/27 17:02:55  oliver
 * ComputerPlayer added
 *
 */



package armada;



import java.io.IOException;
import java.util.*;

import org.apache.log4j.Logger;



/**
 * Title:        Armada
 * Description:  Ein einfaches "Schiffe versenken"-Beispiel
 * Copyright:    Copyright (c) 2001
 * @author Oliver Boehm
 * @version 1.0
 */

public class ComputerPlayer extends Player {

    /** needed for Serializable */
    private static final long serialVersionUID = 4051330050051815219L;

    /** where to log */
    private static Logger log = Logger.getLogger(ComputerPlayer.class);

    /** remember the last shot */
    private Shot lastShot = new Shot();

    private Random rand = new Random();

    public ComputerPlayer() {
        name = Config.getInstance().getProperty("computer.name");
    }
    
    public ComputerPlayer(String name) {
        super(name);
        log.info("computer player " + name + " created");
    }

    /**
     * it's the computers turn now
     */
    public Shot turn() {
    	Shot newShot = null;
    	if (lastShot.isHit() && !lastShot.sunkShip()) {
    	    newShot = this.guessNextHit();
    	} else {
    	    lastShot = newShot = this.findEmptyField();
    	}
    	char hit = opponent.getBoard().shootTo(newShot);
    	if ((hit >= '1') && (hit <= '4')) {	// hit ship
    	    lastShot.setHit(hit);
    	    if (lastShot.sunkShip()) {
    	    	opponent.getBoard().markShipEnvironment(lastShot);
    	    }
    	    newShot.setHitsFrom(lastShot);
    	}
        try {
            this.store();
        } catch (IOException e) {
            log.warn("can't store score");
        }
        log.info("computer " + name + " shoots to " + newShot);
    	return newShot;
    }



    /**
     * @return	the shot on a field which was not shot before
     */

    private Shot findEmptyField() {
    	Shot shot = null;
    	do {
    	    shot = new Shot();
    	} while (opponent.getBoard().isInvalidShot(shot));
    	return shot;
    }



    /**
     * @return	a neighbour field from lastShot which may be a candidate
     * 		for the next hit
     */

    private Shot guessNextHit() {

    	try {
    	    //  look if there's a hit in the neighbour hood
    	    if (opponent.getBoard().isHit(lastShot.up()) ||
    	    	    opponent.getBoard().isHit(lastShot.down())) {
    	        Shot shot = findNextVerticalShot();
                log.info(name + ": trying it with " + shot);
                return shot;
    	    } else if (opponent.getBoard().isHit(lastShot.left()) ||
    	    	    opponent.getBoard().isHit(lastShot.right())) {
    	        Shot shot = findNextHorizontalShot();
                log.info(name + ": trying it with " + shot);
                return shot;
    	    }
    
        	//  look for the most free side
        	int v = opponent.getBoard().countFreeVerticalFields(
        							lastShot);
       	    int h = opponent.getBoard().countFreeHorizontalFields(
        							lastShot);
    	    if (v > h) {
                Shot shot = guessNextVerticalHit();
                log.info(name + ": trying it with " + shot);
                return shot;
    	    } else if (v < h) {
    	        Shot shot = guessNextHorizontalHit();
                log.info(name + ": trying it with " + shot);
                return shot;
    	    }
    	} catch (ShotException e) {
    	    System.err.println("unexcpected exception\n" + e);
    	    return null;
    	}

    	//  roll the dice
        Shot shot;
    	if (rand.nextInt(2) == 1) {
    	    shot = guessNextVerticalHit();
    	} else {
    	    shot = guessNextHorizontalHit();
    	}
        log.info(name + ": trying it with " + shot);
        return shot;

    }



    /**
     * @return	the next possible shot in vertical direction
     */

    private Shot findNextVerticalShot() {
    	if (rand.nextInt(2) == 1) {
    	    try {
    	    	return findNextUpShot();
    	    } catch (ShotException ex0) {
    	    	try {
    	    	    return findNextDownShot();
    	    	} catch (ShotException ex1) {
    	    	    System.err.println("internal error\n" + ex1);
    	    	    return null;
    	    	}
    	    }
    	} else {
    	    try {
    	        return findNextDownShot();
    	    } catch (ShotException ex0) {
    	    	try {
    	    	    return findNextUpShot();
    	    	} catch (ShotException ex1) {
    	    	    System.err.println("internal error\n" + ex1);
    	    	    return null;
    	    	}
    	    }
    	}
    }



    /**
     * looks fot the next possible shot in the upper direction
     * @return	next possible shot
     */

    private Shot findNextUpShot() throws ShotException {
    	Shot shot = lastShot.up();
    	while (opponent.getBoard().isHit(shot)) {
    	    shot = shot.up();
    	}
    	if (opponent.getBoard().isInvalidShot(shot)) {
     	    throw new ShotException("no shot found");
    	}
    	return shot;
    }



    /**
     * looks fot the next possible shot in the lower direction
     * @return	next possible shot
     */

    private Shot findNextDownShot() throws ShotException {
    	Shot shot = lastShot.down();
    	while (opponent.getBoard().isHit(shot)) {
    	    shot = shot.down();
    	}
    	if (opponent.getBoard().isInvalidShot(shot)) {
    	    throw new ShotException("no shot found");
    	}
    	return shot;
    }



    /**
     * @return	the next possible shot in horizontal direction
     */

    private Shot findNextHorizontalShot() {
    	if (rand.nextInt(2) == 1) {
    	    try {		  // try it first with the left side
    	        return findNextLeftShot();
    	    } catch (ShotException ex0) {
    	    	try {
    	    	    return findNextRightShot();
    	    	} catch (ShotException ex1) {
    	    	    System.err.println("internal error\n" + ex1);
    	    	    return null;
    	    	}
    	    }
    	} else {
    	    try {		  // try if with the right side
    	        return findNextRightShot();
    	    } catch (ShotException ex0) {
    	    	try {
    	    	    return findNextLeftShot();
    	    	} catch (ShotException ex1) {
    	    	    System.err.println("internal error\n" + ex1);
    	    	    return null;
    	    	}
    	    }
    	}
    }



    /**
     * looks fot the next possible shot in the left direction
     * @return	next possible shot
     */

    private Shot findNextLeftShot() throws ShotException {
    	Shot shot = lastShot.left();
    	while (opponent.getBoard().isHit(shot)) {
    	    shot = shot.left();
    	}
    	if (opponent.getBoard().isInvalidShot(shot)) {
    	    throw new ShotException("no shot found");
    	}
    	return shot;
    }



    /**
     * looks fot the next possible shot in the right direction
     * @return	next possible shot
     */

    private Shot findNextRightShot() throws ShotException {
    	Shot shot = lastShot.right();
    	while (opponent.getBoard().isHit(shot)) {
    	    shot = shot.right();
    	}
    	if (opponent.getBoard().isInvalidShot(shot)) {
    	    throw new ShotException("no shot found");
    	}
    	return shot;
    }



    /**
     * guessNextVerticalHit
     * looks for the next possible shot in vertical direction.
     *
     * @return	next possible shot
     */

    private Shot guessNextVerticalHit() {
    	//  look for the most free side
    	try {
    	    int up = opponent.getBoard().countFreeFieldsUp(lastShot);
    	    int down = opponent.getBoard().countFreeFieldsDown(lastShot);
	    if (up > down) {
	        return nextShotUp();
	    } else if (up < down) {
	        return nextShotDown();
	    }

	    //  roll the dice
	    if (rand.nextInt(2) == 1) {
	        return nextShotUp();
	    } else {
	        return nextShotDown();
	    }
    	} catch (ShotException e) {
    	    System.err.println("internal error\n" + e);
    	    return null;
    	}
    }

    private Shot nextShotUp() throws ShotException {
    	Shot shot = lastShot.up();
    	while (opponent.getBoard().isInvalidShot(shot)) {
    	    shot = shot.up();
    	}
    	return shot;
    }

    private Shot nextShotDown() throws ShotException {
    	Shot shot = lastShot.down();
    	while (opponent.getBoard().isInvalidShot(shot)) {
    	    shot = shot.down();
    	}
    	return shot;
    }



    /**
     * guessNextHorizontalHit
     * looks for the next possible shot in vertical direction.
     *
     * @return	next possible shot
     */

    private Shot guessNextHorizontalHit() {
    	try {
	    //  look for the most free side
	    int left = opponent.getBoard().countFreeFieldsLeft(lastShot);
	    int right = opponent.getBoard().countFreeFieldsRight(lastShot);
	    if (left > right) {
	        return nextShotLeft();
	    } else if (left < right) {
	        return nextShotRight();
	    }

	    //  roll the dice
	    if (rand.nextInt(2) == 1) {
	        return nextShotLeft();
	    } else {
	        return nextShotRight();
	    }
    	} catch (ShotException e) {
    	    System.err.println("internal error\n" + e);
    	    return null;
    	}
    }

    private Shot nextShotLeft() throws ShotException {
    	Shot shot = lastShot.left();
    	while (opponent.getBoard().isInvalidShot(shot)) {
    	    shot = shot.left();
    	}
    	return shot;
    }

    private Shot nextShotRight() throws ShotException {
    	Shot shot = lastShot.right();
    	while (opponent.getBoard().isInvalidShot(shot)) {
    	    shot = shot.right();
    	}
    	return shot;
    }

    /**
     * Let the computer play again another computer.
     */
    public static void main(String[] args) {
    	Player player1 = new ComputerPlayer("Computer 1");
    	Player player2 = new ComputerPlayer("Computer 2");
        play(player1, player2);
    }
}
