/*
 * $Id: Player.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 *
 * Copyright (c) 2001 Oliver Boehm (mailto:boehm@2xp.de)
 * http://www.2xp.de
 * All rights reserved.
 *
 * -----------------------------------------
 *
 * $Author: boehm $
 * $Date: 2005/06/24 13:56:04 $
 * $Revision: 1.1 $
 *
 * $Log: Player.java,v $
 * Revision 1.1  2005/06/24 13:56:04  boehm
 * Einspielen der Beispiele und Uebungen inkl. HTML-Seiten
 *
 * Revision 1.9  2002/04/18 16:40:24  oliver
 * conflict merged
 *
 * Revision 1.8  2002/04/18 16:33:07  stan
 * javadoc comment for gameOver() inserted
 *
 * Revision 1.7  2001/12/27 16:21:33  oliver
 * HumanPlayer and Player.main() added
 *
 * Revision 1.6  2001/12/26 10:43:42  oliver
 * class ComputerPlayer added and made Player abstarct
 *
 * Revision 1.5  2001/12/26 09:50:18  oliver
 * computer plays against computer
 *
 * Revision 1.4  2001/12/23 22:54:46  oliver
 * main() added to Player and tested intensiv
 *
 * Revision 1.2  2001/12/22 20:52:23  oliver
 * AllTests added
 *
 * Revision 1.1.1.1  2001/10/06 15:23:10  oliver
 * imported sources
 */



package armada;

import java.io.*;

import org.apache.log4j.Logger;






/**
 * Title:        Armada
 * Description:  Ein einfaches "Schiffe versenken"-Beispiel
 * Copyright:    Copyright (c) 2001
 * Company:      2XP
 * @author Oliver Boehm
 * @version 1.0
 */

abstract public class Player implements Serializable {

    private static final long serialVersionUID = 1L;

    /** where to log */
    private static Logger log = Logger.getLogger(Player.class);

    /** the other player */
    protected Player opponent = null;

    /** the player's board */
    protected Board board = new Board();
    
    /** the player's name */
    protected String name = null;

    
    
    public Player() {
        this("anonymous");
    }
    
    public Player(String name) {
        this.name = name;
        log.info("player " + name + " created");
    }
    


    /**
     * register the other player
     * @param	opponent  the other player
     */

    public void setOpponent(Player opponent) {
    	this.opponent = opponent;
    }



    /**
     * @return	the board
     */

    public Board getBoard() {
    	return this.board;
    }



    /**
     * it's the players turn now
     */

    public abstract Shot turn();

    
    
    public void store() throws IOException {
        OutputStream ostream = new FileOutputStream("board." + name);
        store(new ObjectOutputStream(ostream));
        ostream.close();
    }
    
    /**
     * The opponent is set to null to avoid recursion during writing the
     * different attributes.
     * 
     * @param oostream
     * @throws IOException
     */
    private void store(ObjectOutputStream oostream) throws IOException {
        Player opponent = this.opponent;
        this.opponent = null;
        oostream.writeObject(this);
        this.opponent = opponent;
    }
    
    public Player load() throws IOException {
        InputStream istream = new FileInputStream("board." + name);
        Player player = load(new ObjectInputStream(istream));
        istream.close();
        return player;
    }
    
    private Player load(ObjectInputStream oistream) throws IOException {
        try {
            return (Player) oistream.readObject();
        } catch (ClassNotFoundException e) {
            throw new IOException("unexpected " + e);
        }
    }
    
    public void unload() {
        File file = new File("board." + name);
        file.delete();
    }



    /**
     * Is the game over or not?
     *
     * @return  true (game over) or false (game not over)
     */

    public boolean gameOver() {
    	if ((board.getShipIntactItems() == 0) ||
    		(opponent.getBoard().getShipIntactItems() == 0)) {
    	    return true;
    	} else {
    	    return false;
    	}
    }
    
    
    
    public boolean hasWon() {
    	return (opponent.getBoard().getShipIntactItems() == 0);
    }
    
    
    
    public String toString() {
        return name;
    }
    
    
    
    public boolean equals(Object other) {
        if (other instanceof Player) {
            return equals((Player) other);
        } else {
            return false;
        }
    }

    public boolean equals(Player other) {
        return this.name.equals(other.name) && this.board.equals(other.board);
    }


    public static void main(String[] args) {
    	Player player1 = new HumanPlayer();
    	Player player2 = new ComputerPlayer();
        try {
            player1 = player1.load();
            player2 = player2.load();
            System.out.println("OLD GAME restored");
        } catch (IOException e) {
            System.out.println("beginning NEW GAME\n");
        }
    	play(player1, player2);
        player1.unload();
        player2.unload();
    }

    public static void play(Player player1, Player player2) {
        player1.setOpponent(player2);
        player2.setOpponent(player1);
        BoardView view = new BoardView (player2.getBoard(),
    				        player1.getBoard());
        view.print();
        while (!player1.gameOver()) {
    	    Shot shot1 = player1.turn();
    	    if (player2.gameOver()) {
    	    	break;
    	    }
     	    Shot shot2 = player2.turn();
    	    view.print();
    	    view.printShots(shot1, shot2);
    	}
        if (player1.hasWon()) {
    	    System.out.println("\n" + player1 + " hat gewonnen!\n");
    	} else {
    	    System.out.println("\n" + player2 + " hat gewonnen\n");
    	}
    }

}
