/*
 * $Id: Shot.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 *
 * Copyright (c) 2001 Oliver Boehm (mailto:boehm@2xp.de)
 * http://www.2xp.de
 * All rights reserved.
 *
 * -----------------------------------------
 *
 * $Author: boehm $
 * $Date: 2005/06/24 13:56:04 $
 * $Revision: 1.1 $
 *
 * $Log: Shot.java,v $
 * Revision 1.1  2005/06/24 13:56:04  boehm
 * Einspielen der Beispiele und Uebungen inkl. HTML-Seiten
 *
 * Revision 1.6  2001/12/27 19:12:50  oliver
 * ComputerPlayer.main() extended
 *
 * Revision 1.5  2001/12/27 16:21:33  oliver
 * HumanPlayer and Player.main() added
 *
 * Revision 1.4  2001/12/23 22:54:46  oliver
 * main() added to Player and tested intensiv
 *
 * Revision 1.3  2001/12/23 20:39:35  oliver
 * Player.turn() implemented
 *
 * Revision 1.2  2001/12/22 20:02:19  oliver
 * ShotTest object added / Shot extended
 */



package armada;



import java.io.Serializable;
import java.util.*;

import org.apache.log4j.Logger;



public class Shot implements Cloneable, Serializable {

    private static final long serialVersionUID = 3761122734796518962L;
    private static Logger log = Logger.getLogger(Shot.class);
    private int x;
    private int y;
    private int w = 10;
    private int h = 10;
    private char hit = Board.NOTHING;
    
    /** number of hits */
    private int noHits = 0;
    
    /** initialize Random class */
    static Random rand = new Random();
    
    
    
    /**
     * generates a new shot with random x/y coordinates
     */    
    public Shot() {
        init();
        x = rand.nextInt(w) + 1;
        y = rand.nextInt(h) + 1;
        log.debug("new shot " + this);
    }
    
    
    
    /**
     * @param	x   x-coordinate
     * @param	y   y-coordinate
     */

    public Shot(int x, int y) throws ShotException {
        init();
    	this.x = x;
    	this.y = y;
    	if ((x < 0) || (x > (w+1)) || (y < 0) || (y > (h+1))) {
    	    throw new ShotException("invalid coordinates " + x + "," + y);
    	}
        log.debug("new shot " + this);
    }
    
    private void init() {
        w = Config.getInstance().getBoardWidth();
        h = Config.getInstance().getBoardHeight();
    }
    
    
    
    
    /**
     * @param	cx  cx-coordinate (from 'A' to 'j')
     * @param	y   y-coordinate (from 0 to 9)
     */
    
    public Shot(char cx, int y) throws ShotException {
        this((Character.toUpperCase(cx) - 'A' + 1), y + 1);
    }
    
    

    /**
     * @param	s   x/y-coordinates as string (from "A0" to "J9")
     */
    
    public Shot (String s) throws ShotException {
    	this(s.charAt(0), Character.digit(s.charAt(1), 10));
    }
    
    
    
    public Object clone() {
    	try {
    	    return new Shot(x, y);
    	} catch (ShotException e) {
    	    return null;
    	}
    }
    
    
    
    /**
     * compares the given object if it has the same coordinates.
     * Note, that the hit attribute is not used for comparison.
     * 
     * @param	obj the object to compare
     * @return	true if the object has the same coordinates
     */
    public boolean equals(Object obj) {
    	if (this == obj) {
    	    return true;
    	} else if (obj == null || this.getClass() != obj.getClass()) {
    	    return false;
    	}
    	return this.equals((Shot) obj);
    }
    public boolean equals(Shot shot) {
    	return ((x == shot.x) && (y == shot.y));
    }

    
    
    /**
     * @return	x-coordinate
     */
    public int getX() {
    	return x;
    }
    
    
    
    /**
     * @return	y-coordinate
     */
    public int getY() {
    	return y;
    }
    
    
    
    public char getHit() {
    	return hit;
    }
    
    
    
    /**
     * @return	the next upper shot
     */
    public Shot up() throws ShotException {
    	return new Shot(x, y-1);
    }
    
    
    
    /**
     * @return	one shot down
     */
    public Shot down() throws ShotException {
    	return new Shot(x, y+1);
    }
    
    
    
    /**
     * @return	one shot left
     */
    
    public Shot left() throws ShotException {
    	return new Shot(x-1,y);
    }
    
    
    
    /**
     * @return	one shot right
     */
    
    public Shot right() throws ShotException {
    	return new Shot(x+1,y);
    }
    
    
    
    /**
     * set hit and increase the number of hits
     */
    
    public void setHit(char hit) {
    	this.hit = hit;
    	this.noHits++;
    }
    
    
    
    public void setHitsFrom(Shot other) {
    	this.hit = other.hit;
    	this.noHits = other.noHits;
    }
    
    
    
    /**
     * @return	true, if hit is '1', ..., '4'
     *   	false otherwise
     */
    
    public boolean isHit() {
    	switch (hit) {
    	    case '1':
    	    case '2':
    	    case '3':
    	    case '4':	return true;
    	    default:	return false;
    	}
    }
    
    
    
    /**
     * If the number of hits is the same as the hit than the ship is
     * sunk.
     * 
     * @return	true if ship has no free items
     */
    
    public boolean sunkShip() {
    	return (Character.digit(hit, 10) == noHits);
    }
    
    
    
    public String toString() {
        return "" + ((char)('A' + x)) + y;
    }

}

